@@ -4,6 +4,7 @@ from collections import defaultdict, Counter
 import queue
 
 from mlagents_envs.base_env import (
+    ActionTuple,
     DecisionSteps,
     DecisionStep,
     TerminalSteps,
@@ -16,6 +17,7 @@ from mlagents_envs.side_channel.stats_side_channel import (
 from mlagents.trainers.trajectory import Trajectory, AgentExperience
 from mlagents.trainers.policy import Policy
 from mlagents.trainers.action_info import ActionInfo, ActionInfoOutputs
+from mlagents.trainers.torch.action_log_probs import LogProbsTuple
 from mlagents.trainers.stats import StatsReporter
 from mlagents.trainers.behavior_id_utils import get_global_agent_id
 
@@ -129,21 +131,24 @@ class AgentProcessor:
             done = terminated  # Since this is an ongoing step
             interrupted = step.interrupted if terminated else False
             # Add the outputs of the last eval
-            action = stored_take_action_outputs["action"][idx]
-            if self.policy.use_continuous_act:
-                action_pre = stored_take_action_outputs["pre_action"][idx]
-            else:
-                action_pre = None
-            action_probs = stored_take_action_outputs["log_probs"][idx]
+            stored_actions = stored_take_action_outputs["action"]
+            action_tuple = ActionTuple(
+                continuous=stored_actions.continuous[idx],
+                discrete=stored_actions.discrete[idx],
+            )
+            stored_action_probs = stored_take_action_outputs["log_probs"]
+            log_probs_tuple = LogProbsTuple(
+                continuous=stored_action_probs.continuous[idx],
+                discrete=stored_action_probs.discrete[idx],
+            )
             action_mask = stored_decision_step.action_mask
             prev_action = self.policy.retrieve_previous_action([global_id])[0, :]
             experience = AgentExperience(
                 obs=obs,
                 reward=step.reward,
                 done=done,
-                action=action,
-                action_probs=action_probs,
-                action_pre=action_pre,
+                action=action_tuple,
+                action_probs=log_probs_tuple,
                 action_mask=action_mask,
                 prev_action=prev_action,
                 interrupted=interrupted,
