@@ -5,14 +5,15 @@ import json
 import attr
 import cattr
 
-from mlagents.tf_utils import tf
+from mlagents.torch_utils import torch
+from mlagents.tf_utils import tf, is_available as tf_is_available
 from mlagents_envs.logging_util import get_logger
 from mlagents.trainers import __version__
 from mlagents.trainers.exception import TrainerError
 
 logger = get_logger(__name__)
 
-STATUS_FORMAT_VERSION = "0.1.0"
+STATUS_FORMAT_VERSION = "0.2.0"
 
 
 class StatusType(Enum):
@@ -26,7 +27,8 @@ class StatusType(Enum):
 class StatusMetaData:
     stats_format_version: str = STATUS_FORMAT_VERSION
     mlagents_version: str = __version__
-    tensorflow_version: str = tf.__version__
+    torch_version: str = torch.__version__
+    tensorflow_version: str = tf.__version__ if tf_is_available() else -1
 
     def to_dict(self) -> Dict[str, str]:
         return cattr.unstructure(self)
@@ -49,6 +51,10 @@ class StatusMetaData:
             logger.warning(
                 "Tensorflow checkpoint was saved with a different version of Tensorflow. Model may not resume properly."
             )
+        if self.torch_version != other.torch_version:
+            logger.warning(
+                "PyTorch checkpoint was saved with a different version of PyTorch. Model may not resume properly."
+            )
 
 
 class GlobalTrainingStatus:
