@@ -2,15 +2,16 @@ from typing import List, NamedTuple
 import numpy as np
 
 from mlagents.trainers.buffer import AgentBuffer
+from mlagents_envs.base_env import ActionTuple
+from mlagents.trainers.torch.action_log_probs import LogProbsTuple
 
 
 class AgentExperience(NamedTuple):
     obs: List[np.ndarray]
     reward: float
     done: bool
-    action: np.ndarray
-    action_probs: np.ndarray
-    action_pre: np.ndarray  # TODO: Remove this
+    action: ActionTuple
+    action_probs: LogProbsTuple
     action_mask: np.ndarray
     prev_action: np.ndarray
     interrupted: bool
@@ -105,14 +106,16 @@ class Trajectory(NamedTuple):
 
             agent_buffer_trajectory["masks"].append(1.0)
             agent_buffer_trajectory["done"].append(exp.done)
-            # Add the outputs of the last eval
-            if exp.action_pre is not None:
-                actions_pre = exp.action_pre
-                agent_buffer_trajectory["actions_pre"].append(actions_pre)
 
-            # value is a dictionary from name of reward to value estimate of the value head
-            agent_buffer_trajectory["actions"].append(exp.action)
-            agent_buffer_trajectory["action_probs"].append(exp.action_probs)
+            # Adds the log prob and action of continuous/discrete separately
+            agent_buffer_trajectory["continuous_action"].append(exp.action.continuous)
+            agent_buffer_trajectory["discrete_action"].append(exp.action.discrete)
+            agent_buffer_trajectory["continuous_log_probs"].append(
+                exp.action_probs.continuous
+            )
+            agent_buffer_trajectory["discrete_log_probs"].append(
+                exp.action_probs.discrete
+            )
 
             # Store action masks if necessary. Note that 1 means active, while
             # in AgentExperience False means active.
@@ -122,10 +125,11 @@ class Trajectory(NamedTuple):
             else:
                 # This should never be needed unless the environment somehow doesn't supply the
                 # action mask in a discrete space.
-                agent_buffer_trajectory["action_mask"].append(
-                    np.ones(exp.action_probs.shape, dtype=np.float32), padding_value=1
-                )
 
+                action_shape = exp.action.discrete.shape
+                agent_buffer_trajectory["action_mask"].append(
+                    np.ones(action_shape, dtype=np.float32), padding_value=1
+                )
             agent_buffer_trajectory["prev_action"].append(exp.prev_action)
             agent_buffer_trajectory["environment_rewards"].append(exp.reward)
 
